#include "stdafx.h"
#include "AOTMessages.h"
#include "AOTTools.h"
#include "MainHelperQ.h"

#pragma comment( lib, "psapi.lib" )

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

Helper m_Helper;

CObArray m_MonitorArray;

HICON GetIconFromHWnd(HWND hWnd, BOOL bSmallIcon)
{
    HICON hIcon = (HICON)::SendMessage(hWnd, WM_GETICON, bSmallIcon ? ICON_SMALL : ICON_BIG, 0);                   
    if (!hIcon)
    {
        TCHAR szPath[255];

        DWORD dwProcessId;
        GetWindowThreadProcessId(hWnd, &dwProcessId);
        HANDLE process = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, dwProcessId);
        GetModuleFileNameEx(process, NULL, szPath, 255);

        UINT uFlags = SHGFI_ICON | SHGFI_USEFILEATTRIBUTES;

        if (bSmallIcon)
            uFlags |= SHGFI_SMALLICON;
        else
            uFlags |= SHGFI_LARGEICON;

        SHFILEINFO sfi;
        SHGetFileInfo(szPath, FILE_ATTRIBUTE_NORMAL, &sfi, sizeof(SHFILEINFO), uFlags);

        hIcon = sfi.hIcon;
    }

    return hIcon;
}

// CMonitor

// constucts a monitor class not attached to any handle
CMonitor::CMonitor() : m_hMonitor(NULL)
{
}

// copy constructor
CMonitor::CMonitor(const CMonitor& monitor)
{
    m_hMonitor = (HMONITOR)monitor;
}

CMonitor::~CMonitor()
{
}

void CMonitor::Attach(const HMONITOR hMonitor)
{
    m_hMonitor = hMonitor;
}

HMONITOR CMonitor::Detach()
{
    HMONITOR hMonitor = m_hMonitor;
    m_hMonitor = NULL;
    return hMonitor;
}

void CMonitor::GetMonitorRect(LPRECT lprc) const
{
    MONITORINFO mi;
    RECT rc;

    mi.cbSize = sizeof(mi);
    ::GetMonitorInfo(m_hMonitor, &mi);
    rc = mi.rcMonitor;

    ::SetRect(lprc, rc.left, rc.top, rc.right, rc.bottom);
}

//
// these methods return true if any part of the item intersects the monitor rect
BOOL CMonitor::IsOnMonitor(const POINT pt) const
{
    CRect rect;
    GetMonitorRect(rect);

    return rect.PtInRect(pt);
}

BOOL CMonitor::IsOnMonitor(const CWnd* pWnd) const
{
    CRect rect;
    GetMonitorRect(rect);

    ASSERT(::IsWindow(pWnd->GetSafeHwnd()));
    CRect wndRect;
    pWnd->GetClientRect(&wndRect);
    pWnd->ClientToScreen(&wndRect);

    return rect.IntersectRect(rect, wndRect);
}

BOOL CMonitor::IsOnMonitor(const LPRECT lprc) const
{
    CRect rect;
    GetMonitorRect(rect);

    return rect.IntersectRect(rect, lprc);
}

void CMonitor::CenterRectToMonitor(LPRECT lprc, const BOOL UseWorkAreaRect) const
{
    int w = lprc->right - lprc->left;
    int h = lprc->bottom - lprc->top;

    CRect rect;
    if (UseWorkAreaRect)
        GetWorkAreaRect(&rect);
    else
        GetMonitorRect(&rect);

    lprc->left = rect.left + (rect.Width() - w) / 2;
    lprc->top = rect.top + (rect.Height() - h) / 2;
    lprc->right = lprc->left + w;
    lprc->bottom = lprc->top + h;
}

void CMonitor::CenterWindowToMonitor(CWnd* const pWnd, const BOOL UseWorkAreaRect) const
{
    ASSERT(pWnd);
    ASSERT(::IsWindow(pWnd->m_hWnd));

    CRect rect;
    pWnd->GetWindowRect(&rect);
    CenterRectToMonitor(&rect, UseWorkAreaRect);
    CRect monRect;
    if (UseWorkAreaRect)
        GetWorkAreaRect(&monRect);
    else
        GetMonitorRect(&monRect);
    CRect sRect(rect);
    ::IntersectRect(&rect, &sRect, &monRect);

    pWnd->SetWindowPos(NULL, 0, 0, rect.Width(), rect.Height(), SWP_NOREPOSITION | SWP_NOZORDER | SWP_NOACTIVATE);
    pWnd->SetWindowPos(NULL, rect.left, rect.top, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);
}

//
// the work area does not include the start bar
void CMonitor::GetWorkAreaRect(LPRECT lprc) const
{
    MONITORINFO mi;
    RECT rc;

    mi.cbSize = sizeof(mi);
    ::GetMonitorInfo(m_hMonitor, &mi);
    rc = mi.rcWork;

    ::SetRect(lprc, rc.left, rc.top, rc.right, rc.bottom);
}

void DeleteMonitors()
{
    for (int i = 0; i < m_MonitorArray.GetSize(); i++)
    {
        CMonitor* pMonitor = (CMonitor*)m_MonitorArray.GetAt(i);
        m_MonitorArray.SetAt(i, NULL);
        delete pMonitor;
    }
    m_MonitorArray.SetSize(0);
}

void RefreshMonitors()
{
    DeleteMonitors();
    m_MonitorArray.SetSize(::GetSystemMetrics(SM_CMONITORS));
    ADDMONITOR addMonitor;
    addMonitor.currentIndex = 0;
    ::EnumDisplayMonitors(NULL, NULL, AddMonitorsCallBack, (LPARAM)&addMonitor);
}

BOOL CALLBACK AddMonitorsCallBack(HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor, LPARAM dwData)
{
    LPADDMONITOR pAddMonitor = (LPADDMONITOR)dwData;

    CMonitor* pMonitor = new CMonitor;
    pMonitor->Attach(hMonitor);

    m_MonitorArray.SetAt(pAddMonitor->currentIndex, pMonitor);
    pAddMonitor->currentIndex++;
    CString tmp;
    CRect monitorRect;
    pMonitor->GetMonitorRect(&monitorRect);
    tmp.Format(AOT_DISPLAY_MSG, pAddMonitor->currentIndex);
    pMonitor->AOT_DISP = ::RegisterWindowMessage(tmp);

    return TRUE;
}
