#include "StdAfx.h"
#include "Registry.h"
#include "resource.h"
#include "calendar.h"
#include "builddesktopimage.h"
#include "MainHelperQ.h"
#include "GdiplusWrapper.h"
#include "AOTTools.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern int GetWPPositionByImageSize(CString cstrImagePath, UINT iHorzRes, UINT iVertRes);

CBuildDesktopImage::CBuildDesktopImage()
{
    hBitmap = NULL;
    m_nYPos = m_nXPos = 100;
    m_nTransparency = 125;
    ReadRegistry();
}

CBuildDesktopImage::~CBuildDesktopImage(void)
{
}

void CBuildDesktopImage::SetHWnd(HWND hWnd)
{
    hWnd = hWnd;
}

BOOL CBuildDesktopImage::SaveBitmap(HDC hDC, HBITMAP hBitmap, CString szPath)
{
    HANDLE hFile;
    DWORD dwNumWritten;

    hFile = CreateFile(szPath, GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if (hFile == INVALID_HANDLE_VALUE)
        return false;

    BITMAP Bm;
    BITMAPINFO BitInfo;
    ZeroMemory(&BitInfo, sizeof(BITMAPINFO));
    BitInfo.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    BitInfo.bmiHeader.biBitCount = 0;

    if (!::GetDIBits(hDC, hBitmap, 0, 0, NULL, &BitInfo, DIB_RGB_COLORS))
        return (false);

    Bm.bmHeight = BitInfo.bmiHeader.biHeight;
    Bm.bmWidth = BitInfo.bmiHeader.biWidth;

    BITMAPFILEHEADER BmHdr;

    BmHdr.bfType = 0x4d42;   // 'BM' WINDOWS_BITMAP_SIGNATURE
    BmHdr.bfSize = (((3 * Bm.bmWidth + 3) & ~3) * Bm.bmHeight) + sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
    BmHdr.bfReserved1 = BmHdr.bfReserved2 = 0;
    BmHdr.bfOffBits = (DWORD) sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);

    BitInfo.bmiHeader.biCompression = 0;
    // Writing Bitmap File Header ////
    WriteFile(hFile, &BmHdr, sizeof(BITMAPFILEHEADER), &dwNumWritten, NULL);

    WriteFile(hFile, &BitInfo.bmiHeader, sizeof(BITMAPINFOHEADER), &dwNumWritten, NULL);

    BYTE* pData = new BYTE[BitInfo.bmiHeader.biSizeImage + 5];
    if (!::GetDIBits(hDC, hBitmap, 0, Bm.bmHeight, pData, &BitInfo, DIB_RGB_COLORS))
        return (false);
    if (pData != NULL)
        WriteFile(hFile, pData, BitInfo.bmiHeader.biSizeImage, &dwNumWritten, NULL);

    CloseHandle(hFile);
    delete (pData);

    return TRUE;
}

void CBuildDesktopImage::ReadRegistry()
{
    m_szFontName = CRegistry::GetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Font Name"), _T("Monotype Corsiva"));
    m_nFontSize = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Text Size"), 24);
    m_crTextColor = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Text Color"), (INT)RGB(0, 255, 0));
    m_crBackColor = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Back Ground Color"), (INT)RGB(255, 255, 255));
    m_nTransparency = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Transparency"), 88);
    m_nBackgroundTransparency = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("BackgroundTransparency"), 0);
    m_weekStartOnSunday = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("WeekStartOnSunday"), 0);
    m_enableCalendar = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("EnableCalendar"), 0);
    m_nXPos = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("StartX"), 100);
    m_nYPos = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("StartY"), 100);
    Language = CRegistry::GetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Language"), _T(""));
}

void CBuildDesktopImage::WriteRegistry()
{
    CString szSection(_T("Configuration"));
    CRegistry::SetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Font Name"), m_szFontName);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Text Size"), m_nFontSize);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Text Color"), m_crTextColor);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Back Ground Color"), m_crBackColor);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Transparency"), m_nTransparency);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("BackgroundTransparency"), m_nBackgroundTransparency);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("WeekStartOnSunday"), m_weekStartOnSunday);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("EnableCalendar"), m_enableCalendar);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("StartX"), m_nXPos);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("StartY"), m_nYPos);
    CRegistry::SetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Language"), Language);
}

HBITMAP Create24BPPDIBSection(HDC hDC, int iWidth, int iHeight)
{
    BITMAPINFO bmi;
    HBITMAP hbm;
    LPBYTE pBits;

    // Initialize header to 0s.
    ZeroMemory(&bmi, sizeof(bmi));

    // Fill out the fields you care about.
    bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    bmi.bmiHeader.biWidth = iWidth;
    bmi.bmiHeader.biHeight = iHeight;
    bmi.bmiHeader.biPlanes = 1;
    bmi.bmiHeader.biBitCount = 24;
    bmi.bmiHeader.biCompression = BI_RGB;

    // Create the surface.
    hbm = CreateDIBSection(hDC, &bmi, DIB_RGB_COLORS, (void**)&pBits, NULL, 0);

    return(hbm);
}

BOOL BitmapsCompatible(LPBITMAP lpbm1, LPBITMAP lpbm2)
{
    if (lpbm1->bmBitsPixel != lpbm2->bmBitsPixel)
        return FALSE;
    if (lpbm1->bmPlanes != lpbm2->bmPlanes)
        return FALSE;
    if (lpbm1->bmWidth != lpbm2->bmWidth)
        return FALSE;
    if (lpbm1->bmHeight != lpbm2->bmHeight)
        return FALSE;
    return TRUE;
}

BOOL BlendImages(HBITMAP hbmSrc1, HBITMAP hbmSrc2, HBITMAP hbmDst, DWORD dwWeight1, COLORREF m_crBackColor, BOOL backgroundTransparency)
{
    BITMAP bmSrc1, bmSrc2, bmDst;
    RGBTRIPLE* lprgbSrc1, * lprgbSrc2, * lprgbDst;
    DWORD dwWidthBytes, dwWeight2;
    int x, y;

    // Only values between 0 and 255 are valid.
    if (dwWeight1 > 255)
        return FALSE;

    // Get weighting value for second source image.
    dwWeight2 = 255 - dwWeight1;

    // Get information about the surfaces you were passed.
    if (!GetObject(hbmSrc1, sizeof(BITMAP), &bmSrc1))
        return FALSE;
    if (!GetObject(hbmSrc2, sizeof(BITMAP), &bmSrc2))
        return FALSE;
    if (!GetObject(hbmDst, sizeof(BITMAP), &bmDst))
        return FALSE;

    // Make sure you have data that meets your requirements.
    if (!BitmapsCompatible(&bmSrc1, &bmSrc2))
        return FALSE;
    if (!BitmapsCompatible(&bmSrc1, &bmDst))
        return FALSE;
    if (bmSrc1.bmBitsPixel != 24)
        return FALSE;
    if (bmSrc1.bmPlanes != 1)
        return FALSE;
    if (!bmSrc1.bmBits || !bmSrc2.bmBits || !bmDst.bmBits)
        return FALSE;

    dwWidthBytes = bmDst.bmWidthBytes;

    // Initialize the surface pointers.
    lprgbSrc1 = (RGBTRIPLE *)bmSrc1.bmBits;
    lprgbSrc2 = (RGBTRIPLE *)bmSrc2.bmBits;
    lprgbDst = (RGBTRIPLE *)bmDst.bmBits;

    for (y = 0; y < bmDst.bmHeight; y++)
    {
        for (x = 0; x < bmDst.bmWidth; x++)
        {
            if (backgroundTransparency && m_crBackColor == RGB(lprgbSrc1[x].rgbtRed, lprgbSrc1[x].rgbtGreen, lprgbSrc1[x].rgbtBlue))
            {
                lprgbDst[x].rgbtRed = lprgbSrc2[x].rgbtRed;
                lprgbDst[x].rgbtGreen = lprgbSrc2[x].rgbtGreen;
                lprgbDst[x].rgbtBlue = lprgbSrc2[x].rgbtBlue;
            }
            else
            {
                lprgbDst[x].rgbtRed = (BYTE)((((DWORD)lprgbSrc1[x].rgbtRed * dwWeight1) + ((DWORD)lprgbSrc2[x].rgbtRed * dwWeight2)) >> 8);
                lprgbDst[x].rgbtGreen = (BYTE)((((DWORD)lprgbSrc1[x].rgbtGreen * dwWeight1) + ((DWORD)lprgbSrc2[x].rgbtGreen * dwWeight2)) >> 8);
                lprgbDst[x].rgbtBlue = (BYTE)((((DWORD)lprgbSrc1[x].rgbtBlue * dwWeight1) + ((DWORD)lprgbSrc2[x].rgbtBlue * dwWeight2)) >> 8);
            }
        }

        // Move to next scan line.
        lprgbSrc1 = (RGBTRIPLE *)((LPBYTE)lprgbSrc1 + dwWidthBytes);
        lprgbSrc2 = (RGBTRIPLE *)((LPBYTE)lprgbSrc2 + dwWidthBytes);
        lprgbDst = (RGBTRIPLE *)((LPBYTE)lprgbDst + dwWidthBytes);
    }

    return TRUE;
}

BOOL DoAlphaBlend(HDC hdcDest, // Handle to destination DC.
  int nXOriginDest, // X-coord of upper-left corner.
  int nYOriginDest, // Y-coord of upper-left corner.
  int nWidthDest, // Destination width.
  int nHeightDest, // Destination height.
  HDC hdcSrc, // Handle to source DC.
  int nXOriginSrc, // X-coord of upper-left corner.
  int nYOriginSrc, // Y-coord of upper-left corner.
  int nWidthSrc, // Source width.
  int nHeightSrc, // Source height.
  DWORD dwSourceWeight, // Source weighting (between 0 and 255).
  COLORREF m_crBackColor, // Background color
  BOOL backgroundTransparency) // BackgroundTransparency
{
    HDC hdcSrc1 = NULL;
    HDC hdcSrc2 = NULL;
    HDC hdcDst = NULL;
    HBITMAP hbmSrc1 = NULL;
    HBITMAP hbmSrc2 = NULL;
    HBITMAP hbmDst = NULL;
    BOOL bReturn;

    // Create surfaces for sources and destination images.
    hbmSrc1 = Create24BPPDIBSection(hdcDest, nWidthDest, nHeightDest);
    if (!hbmSrc1)
        goto HANDLEERROR;

    hbmSrc2 = Create24BPPDIBSection(hdcDest, nWidthDest, nHeightDest);
    if (!hbmSrc2)
        goto HANDLEERROR;

    hbmDst = Create24BPPDIBSection(hdcDest, nWidthDest, nHeightDest);
    if (!hbmDst)
        goto HANDLEERROR;


    // Create HDCs to hold our surfaces.
    hdcSrc1 = CreateCompatibleDC(hdcDest);
    if (!hdcSrc1)
        goto HANDLEERROR;

    hdcSrc2 = CreateCompatibleDC(hdcDest);
    if (!hdcSrc2)
        goto HANDLEERROR;

    hdcDst = CreateCompatibleDC(hdcDest);
    if (!hdcDst)
        goto HANDLEERROR;


    // Prepare the surfaces for drawing.
    SelectObject(hdcSrc1, hbmSrc1);
    SelectObject(hdcSrc2, hbmSrc2);
    SelectObject(hdcDst, hbmDst);
    SetStretchBltMode(hdcSrc1, COLORONCOLOR);
    SetStretchBltMode(hdcSrc2, COLORONCOLOR);


    // Capture a copy of the source area.
    if (!StretchBlt(hdcSrc1, 0, 0, nWidthDest, nHeightDest, hdcSrc, nXOriginSrc, nYOriginSrc, nWidthSrc, nHeightSrc, SRCCOPY))
        goto HANDLEERROR;

    // Capture a copy of the destination area.
    if (!StretchBlt(hdcSrc2, 0, 0, nWidthDest, nHeightDest, hdcDest, nXOriginDest, nYOriginDest, nWidthDest, nHeightDest, SRCCOPY))
        goto HANDLEERROR;


    // Blend the two source areas to create the destination image.
    bReturn = BlendImages(hbmSrc1, hbmSrc2, hbmDst, dwSourceWeight, m_crBackColor, backgroundTransparency);


    // Clean up objects you do not need any longer.
    // You cannot delete an object that's selected into an
    // HDC so delete the HDC first.
    DeleteDC(hdcSrc1);
    DeleteDC(hdcSrc2);
    DeleteObject(hbmSrc1);
    DeleteObject(hbmSrc2);


    // Display the blended (destination) image to the target HDC.
    if (bReturn)
    {
        BitBlt(hdcDest, nXOriginDest, nYOriginDest, nWidthDest, nHeightDest, hdcDst, 0, 0, SRCCOPY);
    }


    // Clean up the rest of the objects you created.
    DeleteDC(hdcDst);
    DeleteObject(hbmDst);

    return bReturn;

    HANDLEERROR:

    if (hdcSrc1)
        DeleteDC(hdcSrc1);
    if (hdcSrc2)
        DeleteDC(hdcSrc2);
    if (hdcDst)
        DeleteDC(hdcDst);
    if (hbmSrc1)
        DeleteObject(hbmSrc1);
    if (hbmSrc2)
        DeleteObject(hbmSrc2);
    if (hbmDst)
        DeleteObject(hbmDst);

    return FALSE;
}

void CBuildDesktopImage::StartBitmap()
{
    // Delete the last bitmap
    if (hBitmap != NULL)
    {
        DeleteObject(hBitmap);
        hBitmap = NULL;
    }

    // Screen Resolution //
    nVirtualScreenWidth = GetSystemMetrics(SM_CXVIRTUALSCREEN);
    nVirtualScreenHeight = GetSystemMetrics(SM_CYVIRTUALSCREEN);
    hDC = ::GetDC(hWnd);

    RefreshMonitors();

    hMemDC = ::CreateCompatibleDC(hDC);
    hBitmap = CreateCompatibleBitmap(hDC, nVirtualScreenWidth, nVirtualScreenHeight);
    ::SelectObject(hMemDC, hBitmap);

    COLORREF crDeskColor = GetSysColor(COLOR_DESKTOP);
    CBrush FillBrush;
    FillBrush.CreateSolidBrush(crDeskColor);
    CRect Rect(0, 0, nVirtualScreenWidth, nVirtualScreenHeight);
    ::FillRect(hMemDC, &Rect, FillBrush.operator HBRUSH());
}

#define HIMETRIC_INCH 2540

void CBuildDesktopImage::SetImage(CString img, int style, int monitor, int x, int y, int xCount, int yCount)
{
    LONG lWidth = 0,lHeight = 0;

    if (hBitmap)
    {
        // Image Height and Width

        SetStretchBltMode(hMemDC, HALFTONE);

        Graphics* hBitmapDC = new Graphics(hMemDC);
        CMonitor* mon = (CMonitor*)m_MonitorArray.GetAt(monitor);
        CRect ecranTmp, ecran;
        mon->GetMonitorRect(&ecranTmp);
        ecran.left = ecranTmp.left + x * ecranTmp.Width() / xCount;
        ecran.right = ecranTmp.left + (x + 1) * ecranTmp.Width() / xCount;
        ecran.top = ecranTmp.top + y * ecranTmp.Height() / yCount;
        ecran.bottom = ecranTmp.top + (y + 1) * ecranTmp.Height() / yCount;

        Image* hMainBitmap = Image::FromFile(img);
        if (hMainBitmap != NULL)
        {
            lWidth = hMainBitmap->GetWidth();
            lHeight = hMainBitmap->GetHeight();
            if (lWidth > 0 && lHeight > 0)
            {
                CRect rectangle1, rectangle2;

                Bitmap* image2 = new Bitmap(ecran.Width(), ecran.Height(), PixelFormat32bppARGB);
                Graphics* graph2 = Graphics::FromImage(image2);
                SolidBrush* solidBrush = new SolidBrush(GetSysColor(COLOR_DESKTOP));
                CRect Rect(0, 0, ecran.Width(), ecran.Height());
                graph2->FillRectangle(solidBrush, 0, 0, ecran.Width(), ecran.Height());
                delete solidBrush;

                if (style == 5)
                {
                    style = GetWPPositionByImageSize(img, ecran.Width(), ecran.Height());
                }
                if (style == 0)
                {
                    // Tiled
                    rectangle1.top = 0;
                    rectangle1.left = 0;
                    rectangle1.bottom = lHeight;
                    rectangle1.right = lWidth;
                    for (int xPos = 0; xPos < ecran.Width(); xPos += lWidth)
                    {
                        for (int yPos = 0; yPos < ecran.Height(); yPos += lHeight)
                        {
                            graph2->DrawImage(hMainBitmap, xPos, yPos, rectangle1.Width(), rectangle1.Height());
                        }
                    }
                }
                if (style == 1)
                {
                    // Center

                    int imgWidth = lWidth;
                    int imgHeight = lHeight;
                    if (ecran.Height() < lHeight || ecran.Width() < lWidth)
                    {
                        imgWidth = (lWidth * ecran.Height() < lHeight * ecran.Width()) ? lWidth * ecran.Height() / lHeight : lWidth;
                        imgHeight = (lWidth * ecran.Height() < lHeight * ecran.Width()) ? lHeight * imgWidth / lWidth : lHeight;
                    }
                    if (imgWidth > ecran.Width())
                    {
                        imgWidth = ecran.Width();
                        imgHeight = lHeight * ecran.Width() / lWidth;
                    }
                    rectangle1.top = 0;
                    rectangle1.left = 0;
                    rectangle1.bottom = imgHeight;
                    rectangle1.right = imgWidth;

                    graph2->DrawImage(hMainBitmap, (ecran.Width() - rectangle1.Width()) / 2, (ecran.Height() - rectangle1.Height()) / 2, rectangle1.Width(), rectangle1.Height());
                }

                if (style == 2)
                {
                    // Stretch
                    graph2->DrawImage(hMainBitmap, 0, 0, ecran.Width(), ecran.Height());
                } //Stretch

                if (style == 3)
                {
                    // Adjust
                    int imgWidth = ecran.Width();
                    int imgHeight = lHeight* ecran.Width() / lWidth;
                    if (imgHeight > ecran.Height())
                    {
                        imgHeight = ecran.Height();
                        imgWidth = lWidth * ecran.Height() / lHeight;
                    }
                    rectangle1.top = 0;
                    rectangle1.left = 0;
                    rectangle1.bottom = imgHeight;
                    rectangle1.right = imgWidth;
                    if (ecran.Width() > rectangle1.Width())
                    {
                        graph2->DrawImage(hMainBitmap, (ecran.Width() - rectangle1.Width()) / 2, (ecran.Height() - rectangle1.Height()) / 2, rectangle1.Width(), rectangle1.Height());
                    }
                    else
                    {
                        graph2->DrawImage(hMainBitmap, (ecran.Width() - rectangle1.Width()) / 2, (ecran.Height() - rectangle1.Height()) / 2, rectangle1.Width(), rectangle1.Height());
                    }
                } //Adjust

                if (style == 4)
                {
                    // Adjust from outside
                    int imgWidth = ecran.Width();
                    int imgHeight = lHeight* ecran.Width() / lWidth;
                    if (imgHeight < ecran.Height())
                    {
                        imgHeight = ecran.Height();
                        imgWidth = lWidth * ecran.Height() / lHeight;
                    }
                    rectangle1.top = 0;
                    rectangle1.left = 0;
                    rectangle1.bottom = imgHeight;
                    rectangle1.right = imgWidth;
                    if (ecran.Width() > rectangle1.Width())
                    {
                        graph2->DrawImage(hMainBitmap, (ecran.Width() - rectangle1.Width()) / 2, (ecran.Height() - rectangle1.Height()) / 2, rectangle1.Width(), rectangle1.Height());
                    }
                    else
                    {
                        graph2->DrawImage(hMainBitmap, (ecran.Width() - rectangle1.Width()) / 2, (ecran.Height() - rectangle1.Height()) / 2, rectangle1.Width(), rectangle1.Height());
                    }
                } //Adjust from outside

                if (ecran.left >= 0)
                {
                    if (ecran.top >= 0)
                    {
                        hBitmapDC->DrawImage(image2, ecran.left, ecran.top, ecran.Width(), ecran.Height());
                    }
                    else
                    {
                        hBitmapDC->DrawImage(image2, ecran.left, ecran.top, ecran.Width(), ecran.Height());
                        hBitmapDC->DrawImage(image2, ecran.left, nVirtualScreenHeight + ecran.top, ecran.Width(), ecran.Height());
                    }
                }
                else
                {
                    if (ecran.top >= 0)
                    {
                        hBitmapDC->DrawImage(image2, ecran.left, ecran.top, ecran.Width(), ecran.Height());
                        hBitmapDC->DrawImage(image2, nVirtualScreenWidth + ecran.left, ecran.top, ecran.Width(), ecran.Height());
                    }
                    else
                    {
                        hBitmapDC->DrawImage(image2, ecran.left, ecran.top, ecran.Width(), ecran.Height());
                        hBitmapDC->DrawImage(image2, ecran.left, nVirtualScreenHeight + ecran.top, ecran.Width(), ecran.Height());
                        hBitmapDC->DrawImage(image2, nVirtualScreenWidth + ecran.left, ecran.top, ecran.Width(), ecran.Height());
                        hBitmapDC->DrawImage(image2, nVirtualScreenWidth + ecran.left, nVirtualScreenHeight + ecran.top, ecran.Width(), ecran.Height());
                    }
                }
                delete hMainBitmap;
                delete image2;
                delete graph2;
            }
        }
        delete hBitmapDC;
    }
}
void CBuildDesktopImage::StopBitmap()
{
    if (hBitmap)
    {
        C_Calendar* pCalendar = NULL;
        if (m_enableCalendar == TRUE)
        {
            // Making Calendar
            pCalendar = new C_Calendar(m_weekStartOnSunday);
            pCalendar->SetFont(m_szFontName);
            pCalendar->SetTextColor(m_crTextColor);
            pCalendar->SetBackColor(m_crBackColor);
            pCalendar->SetTextSize(m_nFontSize);
            HDC hCalDC = pCalendar->DrawCalendar(hDC);
            DoAlphaBlend(hMemDC, m_nXPos, m_nYPos, pCalendar->GetWidth(), pCalendar->GetHeight(), hCalDC, 0, 0, pCalendar->GetWidth(), pCalendar->GetHeight(), m_nTransparency, m_crBackColor, m_nBackgroundTransparency);
        }

        TCHAR path[MAX_PATH];
        GetModuleFileName(AfxGetInstanceHandle(), path, MAX_PATH);
        TCHAR drive[MAX_PATH];
        TCHAR dir[MAX_PATH];
        _tsplitpath(path, drive, dir, NULL, NULL);
        CString szNewFile;
        szNewFile.Format(_T("%s%sDesktop Calendar.bmp"), drive, dir);

        SaveBitmap(hMemDC, hBitmap, szNewFile);

        if (pCalendar != NULL)
            delete pCalendar;
        pCalendar = NULL;
        DeleteDC(hMemDC);
        DeleteObject(hBitmap);
        hBitmap = NULL;
        hMemDC = NULL;
        ::ReleaseDC(hWnd, hDC);
    }

    DeleteMonitors();
}
