// SettingsDialog.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"       // main symbols
#include "aerosubc.h"
#include "aeroglss.h"
#include "SettingsDialog.h"
#include "registry.h"
#include "desktopCalendar.h"
#include "MainHelperQ.h"
#include "GdiplusWrapper.h"
#include "AboutDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#pragma comment(linker, "/SECTION:.rsrc,rw") 

#define TRANSFER_SIZE 4096

/////////////////////////////////////////////////////////////////////////////
// CStaticCtrl control

CStaticCtrl::CStaticCtrl()
{
}

CStaticCtrl::~CStaticCtrl()
{
}

void CStaticCtrl::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
    int ctrlID = GetDlgCtrlID();
    switch (ctrlID)
    {
        case IDC_PREVIEW:
            SettingsDialog->m_SettingsDialogImagesQueued.DrawPreview();
            break;
        case IDC_PREVIEW2:
            SettingsDialog->m_SettingsDialogQueueImages.DrawPreview();
            break;
        case IDC_PREVIEW3:
            if (SettingsDialog->m_SettingsDialogQueueImages.dialogYesNoAll != NULL)
                SettingsDialog->m_SettingsDialogQueueImages.dialogYesNoAll->DrawPreview();
            break;
        case IDC_SAMPLE_TEXT:
            SettingsDialog->m_SettingsDialogMisc.DrawSampleText();
            break;
    }
}

/////////////////////////////////////////////////////////////////////////////
// CSettingsDialog dialog

CSettingsDialog::CSettingsDialog() : CTabManager(CSettingsDialog::IDD), uiTimer(0)
{
}

CSettingsDialog::~CSettingsDialog()
{
    SettingsDialog = NULL;
}

BEGIN_MESSAGE_MAP(CSettingsDialog, CTabManager)
    ON_MESSAGE(WM_NI_MSG, OnNotifyIconMsg)
    ON_WM_TIMER()
    ON_MESSAGE(WM_HIDEWINDOW, OnHideMainWindow)
    ON_BN_CLICKED(IDC_APPLY, OnBnClickedApply)
    ON_COMMAND(ID_POPUPMENU_ABOUT, OnPopupMenuAbout)
    ON_COMMAND(ID_CONFIGUREDESKTOPCALENDAR, OnShowConfigWindow)
    ON_COMMAND(ID_SETWALLPAPERTONEXTIMAGE, OnSetToNextImage)
    ON_COMMAND(ID_EXITWPQ, OnExit)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSettingsDialog message handlers

BOOL CSettingsDialog::DestroyWindow()
{
    m_TabSetting.DestroyWindow();
    return CDialog::DestroyWindow();
}

void CSettingsDialog::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDOK, m_BtnOk);
    DDX_Control(pDX, IDCANCEL, m_BtnCancel);
    DDX_Control(pDX, IDC_APPLY, m_btnApply);
}
IMPLEMENT_DYNAMIC(CSettingsDialog, CDialog)

BOOL CSettingsDialog::OnInitDialog()
{
    CTabManager::OnInitDialog();

    AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);

    // Initialize or reset dialog controls.
    ResetMainDialog();

    // Display the tray icon.
    m_TrayIcon.Create(this, WM_NI_MSG, _T("Desktop Calendar"), ::LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDR_MAINFRAME)), IDR_NOTIFYICON_MENU);

    m_TabSetting.EnableStackedTabs(FALSE);
    m_TabSetting.AddPage(&m_SettingsDialogQueueImages);
    m_TabSetting.AddPage(&m_SettingsDialogImagesQueued);
    m_TabSetting.AddPage(&m_SettingsDialogMisc);

    m_TabSetting.SetTreeViewMode(TRUE, TRUE);
    m_TabSetting.SetTreeWidth(150);
    m_TabSetting.Create(this, WS_CHILD | WS_VISIBLE | WS_TABSTOP);
    m_TabSetting.ModifyStyleEx(0, WS_EX_CONTROLPARENT);

    CRect rcSheet, rcPage, rectTree;
    GetWindowRect(&rcSheet);
    ScreenToClient(&rcSheet);

    translate();

    m_TabSetting.SetActivePage(0);
    m_TabSetting.GetActivePage()->GetClientRect(rcPage);
    if (rcPage.Width() > rcSheet.Width())
        rcSheet.right += (rcPage.Width() - rcSheet.Width());
    if (rcPage.Height() > rcSheet.Height())
        rcSheet.bottom += (rcPage.Height() - rcSheet.Height());

    m_TabSetting.SetActivePage(1);
    m_TabSetting.GetActivePage()->GetClientRect(rcPage);
    if (rcPage.Width() > rcSheet.Width())
        rcSheet.right += (rcPage.Width() - rcSheet.Width());
    if (rcPage.Height() > rcSheet.Height())
        rcSheet.bottom += (rcPage.Height() - rcSheet.Height());

    m_TabSetting.SetActivePage(2);
    m_TabSetting.GetActivePage()->GetClientRect(rcPage);
    if (rcPage.Width() > rcSheet.Width())
        rcSheet.right += (rcPage.Width() - rcSheet.Width());
    if (rcPage.Height() > rcSheet.Height())
        rcSheet.bottom += (rcPage.Height() - rcSheet.Height());

    m_TabSetting.RefillPageTree();

    m_TabSetting.SetActivePage(0);
    m_TabSetting.GetTabControl()->ShowWindow(SW_HIDE);

    m_TabSetting.GetClientRect(&rcSheet);
    m_TabSetting.MoveWindow(&rcSheet);

    CString cstrCmdLine;

    // Get the command line.
    cstrCmdLine = AfxGetApp()->m_lpCmdLine;

    // If we send a command line of "-noexit", then keep the window up.
    if (cstrCmdLine.CompareNoCase(_T("-noexit")) == 0)
        m_SettingsDialogMisc.iExitAfter = 0;
    else
        PostMessage(WM_HIDEWINDOW);

    // Change the wallpaper.
    if (m_SettingsDialogMisc.iUseTimer == 1 && m_SettingsDialogMisc.iMinutes > 0)
        uiTimer = (UINT)SetTimer(1, m_SettingsDialogMisc.iMinutes * 60000, NULL);
    else
        m_SettingsDialogImagesQueued.ChangeWallpaper(m_SettingsDialogImagesQueued.m_chkRandomize.GetCheck());

    // get time
    time_t ltime;
    time(&ltime);
    // convert to local time
    struct tm* pTime;
    pTime = localtime(&ltime);

    int timerValue = pTime->tm_hour * 60 * 60 * 1000;
    timerValue += pTime->tm_min * 60 * 1000;
    timerValue += pTime->tm_sec * 1000;

    SetTimer(3, 24 * 60 * 60 * 1000 - timerValue, NULL);

    UpdateData(false);

    return TRUE;
}

void CSettingsDialog::translate()
{
    TC_ITEM item;
    CString buf;

    item.mask = TCIF_TEXT;

    m_SettingsDialogQueueImages.translate();
    m_SettingsDialogImagesQueued.translate();
    m_SettingsDialogMisc.translate();

    CPropertyPage* page = m_TabSetting.GetActivePage();
    int i = 0;

    if (page == &m_SettingsDialogQueueImages)
        i = 0;
    else if (page == &m_SettingsDialogImagesQueued)
        i = 1;
    else if (page == &m_SettingsDialogMisc)
        i = 2;

    m_TabSetting.SetActivePage(0);
    buf = m_Helper.translateWindowTextString(&m_SettingsDialogQueueImages, IDD_SETTINGS_QUEUE_IMAGES);
    item.pszText = (LPTSTR)buf.GetString();
    m_TabSetting.GetTabControl()->SetItem(0, &item);
    m_TabSetting.SetActivePage(1);
    buf = m_Helper.translateWindowTextString(&m_SettingsDialogImagesQueued, IDD_SETTINGS_IMAGES_QUEUED);
    item.pszText = (LPTSTR)buf.GetString();
    m_TabSetting.GetTabControl()->SetItem(1, &item);
    m_TabSetting.SetActivePage(2);
    buf = m_Helper.translateWindowTextString(&m_SettingsDialogMisc, IDD_SETTINGS_MISC);
    item.pszText = (LPTSTR)buf.GetString();
    m_TabSetting.GetTabControl()->SetItem(2, &item);

    m_TabSetting.RefillPageTree();

    m_TabSetting.SetActivePage(i);

    m_Helper.translateWindowText(this, IDD_SETTINGS);
    m_Helper.translateDlgItemText(this, IDOK);
    m_Helper.translateDlgItemText(this, IDCANCEL);
    m_Helper.translateDlgItemText(this, IDC_APPLY);
}

void CSettingsDialog::OnOK()
{
    UpdateData(TRUE);
    m_TabSetting.SetRedraw(FALSE);
    // Apply changes, then minimize.
    ApplyChanges(SettingsDialog->m_btnApply.IsWindowEnabled());
    m_TabSetting.SetRedraw(TRUE);
    m_TabSetting.Invalidate();

    SettingsDialog->m_btnApply.EnableWindow(FALSE);
    SendMessage(WM_HIDEWINDOW);
}

void CSettingsDialog::OnCancel()
{
    m_SettingsDialogQueueImages.ResetMainDialog();
    m_SettingsDialogImagesQueued.ResetMainDialog();
    m_SettingsDialogMisc.ResetMainDialog();
    SendMessage(WM_HIDEWINDOW);
}

LRESULT CSettingsDialog::OnNotifyIconMsg(WPARAM wParam, LONG lParam)
{
    return m_TrayIcon.OnTrayNotification(wParam, lParam);
}

void CSettingsDialog::OnTimer(UINT nIDEvent)
{
    CDialog::OnTimer(nIDEvent);

    if (nIDEvent == 1)
    {
        // When the timer is fired, change the wallpaper to the next image.
        m_SettingsDialogImagesQueued.ChangeWallpaper(m_SettingsDialogImagesQueued.m_chkRandomize.GetCheck());
    }
    else if (nIDEvent == 3)
    {
        // When the timer is fired, change the wallpaper to the next image.
        m_SettingsDialogImagesQueued.ChangeWallpaper(m_SettingsDialogImagesQueued.m_chkRandomize.GetCheck());
        KillTimer(3);

        // get time
        time_t ltime;
        time(&ltime);
        // convert to local time
        struct tm* pTime;
        pTime = localtime(&ltime);

        int timerValue = pTime->tm_hour * 60 * 60 * 1000;
        timerValue += pTime->tm_min * 60 * 1000;
        timerValue += pTime->tm_sec * 1000;

        SetTimer(3, 24 * 60 * 60 * 1000 - timerValue, NULL);
    }
}

void CSettingsDialog::OnShowConfigWindow()
{
    CRect trayrect;

    m_TrayIcon.GetTrayWndRect(&trayrect);

    DrawAnimatedRects(IDANI_CAPTION, &trayrect, &rcLastSize);

    MoveWindow(&rcLastSize);

    //Test if image list is not empty, select the second page
    if (m_SettingsDialogImagesQueued.m_lbQList.GetCount() != 0)
        m_TabSetting.SetActivePage(1);
    else
        m_TabSetting.SetActivePage(0);
}

LRESULT CSettingsDialog::OnHideMainWindow(WPARAM wParam, LONG lParam)
{
    RECT rcTray, rcWindow;

    m_TrayIcon.GetTrayWndRect(&rcTray);
    GetWindowRect(&rcWindow);

    rcLastSize = rcWindow;

    MoveWindow(rcTray.left, rcTray.top, 0, 0);

    DrawAnimatedRects(IDANI_CAPTION, &rcWindow, &rcTray);

    return TRUE;
}

void CSettingsDialog::OnSetToNextImage()
{
    ApplyChanges();
}

void CSettingsDialog::ApplyChanges(BOOL bChangeWallpaper /*= TRUE*/)
{
    // Delete all keys under desktopCalendar in the registry.
    CRegistry::RegDelete(_T("Software"), _T(""), _T("Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"));
    m_TabSetting.PressButton(PSBTN_APPLYNOW);
    m_SettingsDialogQueueImages.ApplyChanges();
    m_SettingsDialogMisc.ApplyChanges();
    m_SettingsDialogImagesQueued.ApplyChanges(bChangeWallpaper);

    // If they chose to change the wallpaper every X minutes, kill any
    // previous timers and start a new one based on the minutes entered.
    if (m_SettingsDialogMisc.iUseTimer == 1)
    {
        if (uiTimer != 0)
            KillTimer(uiTimer);

        if (m_SettingsDialogMisc.iMinutes > 0)
            uiTimer = (UINT)SetTimer(1, m_SettingsDialogMisc.iMinutes * 60000, NULL);
    }
    else
    {
        // Or just kill it if they didn't choose to use a timer.
        if (uiTimer != 0)
            KillTimer(uiTimer);
    }

    // Ask them if they want to exit now if they chose to exit after changing wallpaper.
    if (m_SettingsDialogMisc.iExitAfter == 1)
    {
        if (AfxMessageBox(m_Helper.MyLoadString(IDS_EXITAFTERCHANGE), MB_YESNO) == IDYES)
            OnExit();
    }
}

void CSettingsDialog::OnBnClickedApply()
{
    // Apply changes without closing/minimizing.
    UpdateData(true);
    m_TabSetting.SetRedraw(FALSE);
    ApplyChanges();
    m_TabSetting.SetRedraw(TRUE);
    m_TabSetting.Invalidate();
    SettingsDialog->m_btnApply.EnableWindow(FALSE);
}

void CSettingsDialog::OnExit()
{
    CDialog::OnCancel();
}

void CSettingsDialog::OnPopupMenuAbout()
{
    AboutDialog = AboutDialog->getInstance();
}

void CSettingsDialog::ResetMainDialog()
{
    int iScreenWidth, iScreenHeight;
    CString cstrScreenRes, cstrColors;
    CRect rect;

    CDC* pDC = GetDC();  // Get the device context of this window.
    // Get the screen dimensions.
    iScreenWidth = GetDeviceCaps(pDC->m_hDC, HORZRES);
    iScreenHeight = GetDeviceCaps(pDC->m_hDC, VERTRES);

    if (pDC)
        ReleaseDC(pDC);  // Release the device context retrieved earlier.

    // Move the window to the center of the screen.
    GetWindowRect(&rect);
    MoveWindow(iScreenWidth / 2 - (rect.Width()) / 2, iScreenHeight / 2 - (rect.Height()) / 2, (rect.Width()), (rect.Height()));

    SettingsDialog->m_btnApply.EnableWindow(FALSE);

    UpdateData(false);
}

CString CSettingsDialog::PrepareUrlImage(CString imagePath)
{
    // http://systrayplay.free.fr/wallpaper.php5?index=1&resolution=1680x1050
    if (imagePath.Find(_T("http://")) == 0)
    {
        CString imageName = imagePath;
        imageName.Replace(_T(':'), _T('_'));
        imageName.Replace(_T('/'), _T('_'));
        imageName.Replace(_T('.'), _T('_'));
        imageName.Replace(_T('?'), _T('_'));
        imageName.Replace(_T('&'), _T('_'));

        TCHAR applicationPath[MAX_PATH];
        GetModuleFileName(AfxGetInstanceHandle(), applicationPath, MAX_PATH);
        TCHAR drive[MAX_PATH];
        TCHAR dir[MAX_PATH];
        _tsplitpath(applicationPath, drive, dir, NULL, NULL);
        CString tmpDir;
        tmpDir.Format(_T("%s%swallpapers\\"), drive, dir);
        CreateDirectory(tmpDir, NULL);

        CString filePattern;
        filePattern.Format(_T("%s%s.*.jpg"), tmpDir, imageName);

        CTime ti2 = CTime::GetCurrentTime().GetTime();
        CString newFilename;
        newFilename.Format(_T("%s%s.%d.jpg"), tmpDir, imageName, ti2);

        CFileFind finder;

        if (finder.FindFile(filePattern))
        {
            // File already exist
            while (finder.FindNextFile());
            if (finder.GetLength() > 0)
            {
                CString path = finder.GetFilePath();
                CFile cfile;
                cfile.Open(path, CFile::modeReadWrite);

                CFileStatus status;

                cfile.GetStatus(status);

                cfile.Close();

                CTime ti1 = status.m_mtime + 2 * 3600;
                // Reload only if file is older than 2 hours.
                if (ti1 < ti2)
                {
                    CString tempName;
                    tempName.Format(_T("%s%d.tmp"), tmpDir, ti2);
                    if (DownloadTempFile(imagePath, tempName))
                    {
                        if (!CompareFile(path, tempName))
                        {
                            if (!CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("AutoAdd"), FALSE))
                            {
                                DeleteFile(path);
                            }
                            else
                            {
                                SettingsDialog->m_SettingsDialogQueueImages.QueueImage(path);
                                SettingsDialog->m_SettingsDialogImagesQueued.ApplyChanges(FALSE);
                            }
                            MoveFile(tempName, newFilename);
                            return newFilename;
                        }
                        else
                        {
                            DeleteFile(path);
                            MoveFile(tempName, newFilename);
                            return path;
                        }
                    }
                    else
                    {
                        //DeleteFile(tempName);
                        return path;
                    }
                }
                else
                {
                    return path;
                }
            }
            else
            {
                // Don't know why but file doesn't exist.
                if (DownloadTempFile(imagePath, newFilename))
                {
                    return newFilename;
                }
                else
                {
                    return _T("");
                }
            }
        }
        else
        {
            if (DownloadTempFile(imagePath, newFilename))
            {
                return newFilename;
            }
            else
            {
                return _T("");
            }
        }
    }
    else
    {
        return imagePath;
    }
}

BOOL CSettingsDialog::CompareFile(CString file1, CString file2)
{
    CFile cfile1;
    if (!cfile1.Open(file1, CFile::modeRead))
    {
        cfile1.Close();
        return FALSE;
    }

    CFile cfile2;
    if (!cfile2.Open(file2, CFile::modeRead))
    {
        cfile1.Close();
        cfile2.Close();
        return FALSE;
    }

    BYTE pBuf1[TRANSFER_SIZE];
    BYTE pBuf2[TRANSFER_SIZE];
    UINT readFile1 = cfile1.Read(pBuf1, TRANSFER_SIZE);
    UINT readFile2 = cfile2.Read(pBuf2, TRANSFER_SIZE);

    while (readFile1 > 0 && readFile2 > 0)
    {
        if (memcmp(pBuf1, pBuf2, readFile1) != 0)
        {
            cfile1.Close();
            cfile2.Close();
            return FALSE;
        }
        readFile1 = cfile1.Read(pBuf1, TRANSFER_SIZE);
        readFile2 = cfile2.Read(pBuf2, TRANSFER_SIZE);
    }

    cfile1.Close();
    cfile2.Close();
    return TRUE;
}

BOOL CSettingsDialog::DownloadTempFile(CString imagePath, CString tempName)
{
    CDialog dlg;
    dlg.Create(IDD_PROGRESS);

    CRect rect2;
    dlg.CenterWindow();
    dlg.ShowWindow(SW_SHOW);
    dlg.UpdateWindow();

    // Canonicalization of the URL converts unsafe characters into escape character equivalents
    TCHAR canonicalURL[1024];
    DWORD nSize = 1024;
    InternetCanonicalizeUrl(imagePath, canonicalURL, &nSize, ICU_BROWSER_MODE);     

    DWORD options = INTERNET_FLAG_NEED_FILE | INTERNET_FLAG_HYPERLINK | INTERNET_FLAG_RESYNCHRONIZE | INTERNET_FLAG_RELOAD;

    dlg.GetDlgItem(IDC_LLABEL)->SetWindowText(m_Helper.MyLoadString(IDS_OPENCONNECTION));

    HINTERNET hInternet = InternetOpen(_T("DownloaderAgent"), PRE_CONFIG_INTERNET_ACCESS, NULL, NULL, NULL);

    dlg.GetDlgItem(IDC_LLABEL)->SetWindowText(m_Helper.MyLoadString(IDS_CONNECTING));
    HINTERNET hSession = InternetOpenUrl(hInternet, canonicalURL, NULL, NULL, options, 0);
    dlg.GetDlgItem(IDC_LLABEL)->SetWindowText(m_Helper.MyLoadString(IDS_CONNECTIONOK));

    if (hSession)
    {
        HANDLE hFile;
        BYTE pBuf[TRANSFER_SIZE];
        DWORD dwReadSizeOut = TRANSFER_SIZE, dwTotalReadSize = 0, totalSize = 0;

        hFile = CreateFile(tempName, GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
        if (hFile == INVALID_HANDLE_VALUE)
            return FALSE;

        BOOL bRead = HttpQueryInfo(hSession, HTTP_QUERY_CONTENT_LENGTH, &pBuf, &dwReadSizeOut, NULL);
        totalSize = _tstoi((TCHAR *)pBuf);

        ((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->SetRange32(0, totalSize + 1);
        ((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->SetStep(1);
        ((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->StepIt();

        do
        {
            DWORD dwWriteSize, dwNumWritten;
            bRead = InternetReadFile(hSession, pBuf, TRANSFER_SIZE, &dwReadSizeOut);
            dwWriteSize = dwReadSizeOut;

            if (bRead && dwReadSizeOut > 0)
            {
                dwTotalReadSize += dwReadSizeOut;
                WriteFile(hFile, pBuf, dwWriteSize, &dwNumWritten, NULL); 
                // File write error
                if (dwWriteSize != dwNumWritten)
                {
                    CloseHandle(hFile);                 
                    dlg.DestroyWindow();
                    return FALSE;
                }
            }
            else
            {
                if (!bRead)
                {
                    // Error
                    CloseHandle(hFile); 
                    dlg.DestroyWindow();
                    return FALSE;
                }           
                break;
            }
            CString tmp, buf = m_Helper.MyLoadString(IDS_RETRIEVING);
            tmp.Format(buf, dwTotalReadSize, totalSize);
            dlg.GetDlgItem(IDC_LLABEL)->SetWindowText(tmp);
            ((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->SetStep(dwReadSizeOut);
            ((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->StepIt();
        }
        while (1);

        CloseHandle(hFile);
    }
    else
    {
        dlg.DestroyWindow();
        return FALSE;
    }

    dlg.DestroyWindow();
    return TRUE;
}

void CSettingsDialog::SetIcon()
{
    CTime tm = CTime::GetCurrentTime();
    CString date = tm.Format(_T("%A %d %B %Y"));

    CStringArray weekdays;
    CStringArray monthsName;

    CString dayList = m_Helper.MyLoadString(IDS_LLONGDAYLIST);
    int startPos = 0, i = 0;
    while (dayList.Find(_T(","), startPos) != -1 && i < 6)
    {
        CString tmp = dayList.Mid(startPos, dayList.Find(_T(","), startPos) - startPos);
        weekdays.Add(tmp);
        startPos = dayList.Find(_T(","), startPos) + 1;
        i++;
    }
    weekdays.Add(dayList.Mid(startPos));
    CString monthList = m_Helper.MyLoadString(IDS_LMONTHLIST);
    startPos = 0, i = 0;
    while (monthList.Find(_T(","), startPos) != -1 && i < 11)
    {
        CString tmp = monthList.Mid(startPos, monthList.Find(_T(","), startPos) - startPos);
        monthsName.Add(tmp);
        startPos = monthList.Find(_T(","), startPos) + 1;
        i++;
    }
    monthsName.Add(monthList.Mid(startPos));

    date.Replace(_T("Monday"), weekdays[0]);
    date.Replace(_T("Tuesday"), weekdays[1]);
    date.Replace(_T("Wednesday"), weekdays[2]);
    date.Replace(_T("Thursday"), weekdays[3]);
    date.Replace(_T("Friday"), weekdays[4]);
    date.Replace(_T("Saturday"), weekdays[5]);
    date.Replace(_T("Sunday"), weekdays[6]);
    date.Replace(_T("January"), monthsName[0]);
    date.Replace(_T("February"), monthsName[1]);
    date.Replace(_T("March"), monthsName[2]);
    date.Replace(_T("April"), monthsName[3]);
    date.Replace(_T("May"), monthsName[4]);
    date.Replace(_T("June"), monthsName[5]);
    date.Replace(_T("July"), monthsName[6]);
    date.Replace(_T("August"), monthsName[7]);
    date.Replace(_T("September"), monthsName[8]);
    date.Replace(_T("October"), monthsName[9]);
    date.Replace(_T("November"), monthsName[10]);
    date.Replace(_T("December"), monthsName[11]);
    HICON hIcon = ::LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_ICON1 + (tm.GetDay() - 1)));

    if (hIcon)
    {
        m_TrayIcon.SetTooltipText(date);
        m_TrayIcon.SetIcon(hIcon);
    }
}

